library ieee;
use ieee.std_logic_1164.all;

entity SystemeSecurite is
port (Touche: in std_logic_vector(0 to 9); Entree: in std_logic;
   Horl: in std_logic; Alarme: out std_logic);
end entity SystemeSecurite;

architecture ComportementSystemeSecurite of SystemeSecurite is

component CodeurDCB is
port (D: in std_logic_vector(0 to 9);
   Q: out std_logic_vector(0 to 3));
end component CodeurDCB;

component RegDecQuatreBits is
port (D: in std_logic_vector(0 to 3);
   Horl: in std_logic;
   Q: out std_logic_vector(0 to 3));
end component RegDecQuatreBits;

component ComparateurQuatreBits is
port (A, B: in std_logic_vector(0 to 3);
   EG: out std_logic);
end component ComparateurQuatreBits;

component Monostable is
port (Validation, Horl: in std_logic;
   Qsor: buffer std_logic);
end component Monostable;

component RegDecHuitBits is
port (S_L, Horl: in std_logic;
   D: in std_logic_vector(0 to 7);
   Q: buffer std_logic);
end component RegDecHuitBits;

component SelectionCode is
port (EntDecal, Horl: in std_logic;
   Bsor: out std_logic_vector(1 to 4));
end component SelectionCode;

signal DCBsor: std_logic_vector(0 to 3);
signal DecAsor: std_logic_vector(0 to 3);
signal DecBsor: std_logic_vector(0 to 3);
signal MCodeEnt: std_logic_vector(0 to 3);
signal PorteOU: std_logic;
signal CompareGrand: std_logic;
signal DelaiA, DelaiB: std_logic;

begin
PorteOU <= (Touche(0) or Touche(1) or Touche(2) or Touche(3) or Touche(4)
or Touche(5) or Touche(6) or Touche(7) or Touche(8) or Touche(9));

DCB : CodeurDCB
port map (D(0) => Touche(0),D(1) => Touche(1),D(2) => Touche(2),D(3) => Touche(3),
D(4) => Touche(4),D(5) => Touche(5),D(6) => Touche(6),D(7) => Touche(7),D(8) => Touche(8),D(9) => Touche(9),
Q(0) => DCBsor(0),Q(1) => DCBsor(1),Q(2) => DCBsor(2),Q(3) => DCBsor(3));

RegistreDecalageA : RegDecQuatreBits
port map (D(0) => DCBsor(0),D(1) => DCBsor(1),D(2) => DCBsor(2),D(3) => DCBsor(3),
Horl => not DelaiA,Q(0) => DecAsor(0),Q(1) => DecAsor(1),Q(2) => DecAsor(2),Q(3) => DecAsor(3));

RegistreDecalageB : RegDecQuatreBits
port map (D(0) => MCodeEnt(0),D(1) => MCodeEnt(1),D(2) => MCodeEnt(2),D(3) => MCodeEnt(3),
Horl => not DelaiA,Q(0) => DecBsor(0),Q(1) => DecBsor(1),Q(2) => DecBsor(2),Q(3) => DecBsor(3));

ComparateurGrandeurs : ComparateurQuatreBits port map (A => DecAsor,B => DecBsor,EG => CompareGrand);

MSA : Monostable port map (Validation => Entree or PorteOU,Horl => Horl,Qsor => DelaiA);

MSB : Monostable port map (Validation => not DelaiA,Horl => Horl,Qsor => DelaiB);

RegistreDecalageC : RegDecHuitBits
port map (S_L => CompareGrand,Horl =>  DelaiB,D(0) => '0',D(1) => '0',
D(2) => '0',D(3) => '1',D(4) => '0',D(5) => '0',D(6) => '0',D(7) => '0',Q => Alarme);

SelectionCodeA : SelectionCode
port map (EntDecal => CompareGrand,Horl => Entree or PorteOU,Bsor => MCodeEnt);

end architecture ComportementSystemeSecurite;